@@ -70,12 +70,10 @@ Req-sent-unread-response       _CS_REQ_SENT       <response_class>
 
 import email.parser
 import email.message
-import errno
 import http
 import io
 import re
 import socket
-import sys
 import collections.abc
 from urllib.parse import urlsplit
 
@@ -107,15 +105,13 @@ globals().update(http.HTTPStatus.__members__)
 # Mapping status codes to official W3C names
 responses = {v: v.phrase for v in http.HTTPStatus.__members__.values()}
 
+# maximal amount of data to read at one time in _safe_read
+MAXAMOUNT = 1048576
+
 # maximal line length when calling readline().
 _MAXLINE = 65536
 _MAXHEADERS = 100
 
-# Data larger than this will be read in chunks, to prevent extreme
-# overallocation.
-_MIN_READ_BUF_SIZE = 1 << 20
-
-
 # Header name/value ABNF (http://tools.ietf.org/html/rfc7230#section-3.2)
 #
 # VCHAR          = %x21-7E
@@ -177,13 +173,6 @@ def _encode(data, name='data'):
             "if you want to send it encoded in UTF-8." %
             (name.title(), data[err.start:err.end], name)) from None
 
-def _strip_ipv6_iface(enc_name: bytes) -> bytes:
-    """Remove interface scope from IPv6 address."""
-    enc_name, percent, _ = enc_name.partition(b"%")
-    if percent:
-        assert enc_name.startswith(b'['), enc_name
-        enc_name += b']'
-    return enc_name
 
 class HTTPMessage(email.message.Message):
     # XXX The only usage of this method is in
@@ -233,9 +222,8 @@ def _read_headers(fp):
             break
     return headers
 
-def _parse_header_lines(header_lines, _class=HTTPMessage):
-    """
-    Parses only RFC 5322 headers from header lines.
+def parse_headers(fp, _class=HTTPMessage):
+    """Parses only RFC2822 headers from a file pointer.
 
     email Parser wants to see strings rather than bytes.
     But a TextIOWrapper around self.rfile would buffer too many bytes
@@ -244,14 +232,9 @@ def _parse_header_lines(header_lines, _class=HTTPMessage):
     to parse.
 
     """
-    hstring = b''.join(header_lines).decode('iso-8859-1')
-    return email.parser.Parser(_class=_class).parsestr(hstring)
-
-def parse_headers(fp, _class=HTTPMessage):
-    """Parses only RFC 5322 headers from a file pointer."""
-
     headers = _read_headers(fp)
-    return _parse_header_lines(headers, _class)
+    hstring = b''.join(headers).decode('iso-8859-1')
+    return email.parser.Parser(_class=_class).parsestr(hstring)
 
 
 class HTTPResponse(io.BufferedIOBase):
@@ -373,6 +356,9 @@ class HTTPResponse(io.BufferedIOBase):
         # NOTE: RFC 2616, S4.4, #3 says we ignore this if tr_enc is "chunked"
         self.length = None
         length = self.headers.get("content-length")
+
+         # are we using the chunked-style of transfer encoding?
+        tr_enc = self.headers.get("transfer-encoding")
         if length and not self.chunked:
             try:
                 self.length = int(length)
@@ -466,7 +452,6 @@ class HTTPResponse(io.BufferedIOBase):
         return self.fp is None
 
     def read(self, amt=None):
-        """Read and return the response body, or up to the next amt bytes."""
         if self.fp is None:
             return b""
 
@@ -474,25 +459,18 @@ class HTTPResponse(io.BufferedIOBase):
             self._close_conn()
             return b""
 
-        if self.chunked:
-            return self._read_chunked(amt)
-
-        if amt is not None and amt >= 0:
-            if self.length is not None and amt > self.length:
-                # clip the read to the "end of response"
-                amt = self.length
-            s = self.fp.read(amt)
-            if not s and amt:
-                # Ideally, we would raise IncompleteRead if the content-length
-                # wasn't satisfied, but it might break compatibility.
-                self._close_conn()
-            elif self.length is not None:
-                self.length -= len(s)
-                if not self.length:
-                    self._close_conn()
-            return s
+        if amt is not None:
+            # Amount is given, implement using readinto
+            b = bytearray(amt)
+            n = self.readinto(b)
+            return memoryview(b)[:n].tobytes()
         else:
             # Amount is not given (unbounded read) so we must check self.length
+            # and self.chunked
+
+            if self.chunked:
+                return self._readall_chunked()
+
             if self.length is None:
                 s = self.fp.read()
             else:
@@ -593,25 +571,19 @@ class HTTPResponse(io.BufferedIOBase):
             self.chunk_left = chunk_left
         return chunk_left
 
-    def _read_chunked(self, amt=None):
+    def _readall_chunked(self):
         assert self.chunked != _UNKNOWN
-        if amt is not None and amt < 0:
-            amt = None
         value = []
         try:
-            while (chunk_left := self._get_chunk_left()) is not None:
-                if amt is not None and amt <= chunk_left:
-                    value.append(self._safe_read(amt))
-                    self.chunk_left = chunk_left - amt
+            while True:
+                chunk_left = self._get_chunk_left()
+                if chunk_left is None:
                     break
-
                 value.append(self._safe_read(chunk_left))
-                if amt is not None:
-                    amt -= chunk_left
                 self.chunk_left = 0
             return b''.join(value)
-        except IncompleteRead as exc:
-            raise IncompleteRead(b''.join(value)) from exc
+        except IncompleteRead:
+            raise IncompleteRead(b''.join(value))
 
     def _readinto_chunked(self, b):
         assert self.chunked != _UNKNOWN
@@ -638,39 +610,43 @@ class HTTPResponse(io.BufferedIOBase):
             raise IncompleteRead(bytes(b[0:total_bytes]))
 
     def _safe_read(self, amt):
-        """Read the number of bytes requested.
+        """Read the number of bytes requested, compensating for partial reads.
+
+        Normally, we have a blocking socket, but a read() can be interrupted
+        by a signal (resulting in a partial read).
+
+        Note that we cannot distinguish between EOF and an interrupt when zero
+        bytes have been read. IncompleteRead() will be raised in this
+        situation.
 
         This function should be used when <amt> bytes "should" be present for
         reading. If the bytes are truly not available (due to EOF), then the
         IncompleteRead exception can be used to detect the problem.
         """
-        cursize = min(amt, _MIN_READ_BUF_SIZE)
-        data = self.fp.read(cursize)
-        if len(data) >= amt:
-            return data
-        if len(data) < cursize:
-            raise IncompleteRead(data, amt - len(data))
-
-        data = io.BytesIO(data)
-        data.seek(0, 2)
-        while True:
-            # This is a geometric increase in read size (never more than
-            # doubling out the current length of data per loop iteration).
-            delta = min(cursize, amt - cursize)
-            data.write(self.fp.read(delta))
-            if data.tell() >= amt:
-                return data.getvalue()
-            cursize += delta
-            if data.tell() < cursize:
-                raise IncompleteRead(data.getvalue(), amt - data.tell())
+        s = []
+        while amt > 0:
+            chunk = self.fp.read(min(amt, MAXAMOUNT))
+            if not chunk:
+                raise IncompleteRead(b''.join(s), amt)
+            s.append(chunk)
+            amt -= len(chunk)
+        return b"".join(s)
 
     def _safe_readinto(self, b):
         """Same as _safe_read, but for reading into a buffer."""
-        amt = len(b)
-        n = self.fp.readinto(b)
-        if n < amt:
-            raise IncompleteRead(bytes(b[:n]), amt-n)
-        return n
+        total_bytes = 0
+        mvb = memoryview(b)
+        while total_bytes < len(b):
+            if MAXAMOUNT < len(mvb):
+                temp_mvb = mvb[0:MAXAMOUNT]
+                n = self.fp.readinto(temp_mvb)
+            else:
+                n = self.fp.readinto(mvb)
+            if not n:
+                raise IncompleteRead(bytes(mvb[0:total_bytes]), len(b))
+            mvb = mvb[n:]
+            total_bytes += n
+        return total_bytes
 
     def read1(self, n=-1):
         """Read with at most one underlying system call.  If at least one
@@ -687,8 +663,6 @@ class HTTPResponse(io.BufferedIOBase):
             self._close_conn()
         elif self.length is not None:
             self.length -= len(result)
-            if not self.length:
-                self._close_conn()
         return result
 
     def peek(self, n=-1):
@@ -713,8 +687,6 @@ class HTTPResponse(io.BufferedIOBase):
             self._close_conn()
         elif self.length is not None:
             self.length -= len(result)
-            if not self.length:
-                self._close_conn()
         return result
 
     def _read1_chunked(self, n):
@@ -822,20 +794,6 @@ class HTTPResponse(io.BufferedIOBase):
         '''
         return self.status
 
-
-def _create_https_context(http_version):
-    # Function also used by urllib.request to be able to set the check_hostname
-    # attribute on a context object.
-    context = ssl._create_default_https_context()
-    # send ALPN extension to indicate HTTP/1.1 protocol
-    if http_version == 11:
-        context.set_alpn_protocols(['http/1.1'])
-    # enable PHA for TLS 1.3 connections if available
-    if context.post_handshake_auth is not None:
-        context.post_handshake_auth = True
-    return context
-
-
 class HTTPConnection:
 
     _http_vsn = 11
@@ -897,7 +855,6 @@ class HTTPConnection:
         self._tunnel_host = None
         self._tunnel_port = None
         self._tunnel_headers = {}
-        self._raw_proxy_headers = None
 
         (self.host, self.port) = self._get_hostport(host, port)
 
@@ -910,23 +867,16 @@ class HTTPConnection:
     def set_tunnel(self, host, port=None, headers=None):
         """Set up host and port for HTTP CONNECT tunnelling.
 
-        In a connection that uses HTTP CONNECT tunnelling, the host passed to
-        the constructor is used as a proxy server that relays all communication
-        to the endpoint passed to `set_tunnel`. This done by sending an HTTP
+        In a connection that uses HTTP CONNECT tunneling, the host passed to the
+        constructor is used as a proxy server that relays all communication to
+        the endpoint passed to `set_tunnel`. This done by sending an HTTP
         CONNECT request to the proxy server when the connection is established.
 
-        This method must be called before the HTTP connection has been
+        This method must be called before the HTML connection has been
         established.
 
         The headers argument should be a mapping of extra HTTP headers to send
         with the CONNECT request.
-
-        As HTTP/1.1 is used for HTTP CONNECT tunnelling request, as per the RFC
-        (https://tools.ietf.org/html/rfc7231#section-4.3.6), a HTTP Host:
-        header must be provided, matching the authority-form of the request
-        target provided as the destination for the CONNECT request. If a
-        HTTP Host: header is not provided via the headers argument, one
-        is generated and transmitted automatically.
         """
 
         if self.sock:
@@ -934,15 +884,10 @@ class HTTPConnection:
 
         self._tunnel_host, self._tunnel_port = self._get_hostport(host, port)
         if headers:
-            self._tunnel_headers = headers.copy()
+            self._tunnel_headers = headers
         else:
             self._tunnel_headers.clear()
 
-        if not any(header.lower() == "host" for header in self._tunnel_headers):
-            encoded_host = self._tunnel_host.encode("idna").decode("ascii")
-            self._tunnel_headers["Host"] = "%s:%d" % (
-                encoded_host, self._tunnel_port)
-
     def _get_hostport(self, host, port):
         if port is None:
             i = host.rfind(':')
@@ -958,76 +903,50 @@ class HTTPConnection:
                 host = host[:i]
             else:
                 port = self.default_port
-        if host and host[0] == '[' and host[-1] == ']':
-            host = host[1:-1]
+            if host and host[0] == '[' and host[-1] == ']':
+                host = host[1:-1]
 
         return (host, port)
 
     def set_debuglevel(self, level):
         self.debuglevel = level
 
-    def _wrap_ipv6(self, ip):
-        if b':' in ip and ip[0] != b'['[0]:
-            return b"[" + ip + b"]"
-        return ip
-
     def _tunnel(self):
-        connect = b"CONNECT %s:%d %s\r\n" % (
-            self._wrap_ipv6(self._tunnel_host.encode("idna")),
-            self._tunnel_port,
-            self._http_vsn_str.encode("ascii"))
-        headers = [connect]
+        connect_str = "CONNECT %s:%d HTTP/1.0\r\n" % (self._tunnel_host,
+            self._tunnel_port)
+        connect_bytes = connect_str.encode("ascii")
+        self.send(connect_bytes)
         for header, value in self._tunnel_headers.items():
-            headers.append(f"{header}: {value}\r\n".encode("latin-1"))
-        headers.append(b"\r\n")
-        # Making a single send() call instead of one per line encourages
-        # the host OS to use a more optimal packet size instead of
-        # potentially emitting a series of small packets.
-        self.send(b"".join(headers))
-        del headers
+            header_str = "%s: %s\r\n" % (header, value)
+            header_bytes = header_str.encode("latin-1")
+            self.send(header_bytes)
+        self.send(b'\r\n')
 
         response = self.response_class(self.sock, method=self._method)
-        try:
-            (version, code, message) = response._read_status()
+        (version, code, message) = response._read_status()
 
-            self._raw_proxy_headers = _read_headers(response.fp)
+        if code != http.HTTPStatus.OK:
+            self.close()
+            raise OSError("Tunnel connection failed: %d %s" % (code,
+                                                               message.strip()))
+        while True:
+            line = response.fp.readline(_MAXLINE + 1)
+            if len(line) > _MAXLINE:
+                raise LineTooLong("header line")
+            if not line:
+                # for sites which EOF without sending a trailer
+                break
+            if line in (b'\r\n', b'\n', b''):
+                break
 
             if self.debuglevel > 0:
-                for header in self._raw_proxy_headers:
-                    print('header:', header.decode())
-
-            if code != http.HTTPStatus.OK:
-                self.close()
-                raise OSError(f"Tunnel connection failed: {code} {message.strip()}")
-
-        finally:
-            response.close()
-
-    def get_proxy_response_headers(self):
-        """
-        Returns a dictionary with the headers of the response
-        received from the proxy server to the CONNECT request
-        sent to set the tunnel.
-
-        If the CONNECT request was not sent, the method returns None.
-        """
-        return (
-            _parse_header_lines(self._raw_proxy_headers)
-            if self._raw_proxy_headers is not None
-            else None
-        )
+                print('header:', line.decode())
 
     def connect(self):
         """Connect to the host and port specified in __init__."""
-        sys.audit("http.client.connect", self, self.host, self.port)
         self.sock = self._create_connection(
             (self.host,self.port), self.timeout, self.source_address)
-        # Might fail in OSs that don't implement TCP_NODELAY
-        try:
-            self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
-        except OSError as e:
-            if e.errno != errno.ENOPROTOOPT:
-                raise
+        self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 
         if self._tunnel_host:
             self._tunnel()
@@ -1062,17 +981,18 @@ class HTTPConnection:
             print("send:", repr(data))
         if hasattr(data, "read") :
             if self.debuglevel > 0:
-                print("sending a readable")
+                print("sendIng a read()able")
             encode = self._is_textIO(data)
             if encode and self.debuglevel > 0:
                 print("encoding file using iso-8859-1")
-            while datablock := data.read(self.blocksize):
+            while 1:
+                datablock = data.read(self.blocksize)
+                if not datablock:
+                    break
                 if encode:
                     datablock = datablock.encode("iso-8859-1")
-                sys.audit("http.client.send", self, datablock)
                 self.sock.sendall(datablock)
             return
-        sys.audit("http.client.send", self, data)
         try:
             self.sock.sendall(data)
         except TypeError:
@@ -1092,11 +1012,14 @@ class HTTPConnection:
 
     def _read_readable(self, readable):
         if self.debuglevel > 0:
-            print("reading a readable")
+            print("sendIng a read()able")
         encode = self._is_textIO(readable)
         if encode and self.debuglevel > 0:
             print("encoding file using iso-8859-1")
-        while datablock := readable.read(self.blocksize):
+        while True:
+            datablock = readable.read(self.blocksize)
+            if not datablock:
+                break
             if encode:
                 datablock = datablock.encode("iso-8859-1")
             yield datablock
@@ -1233,7 +1156,7 @@ class HTTPConnection:
                         netloc_enc = netloc.encode("ascii")
                     except UnicodeEncodeError:
                         netloc_enc = netloc.encode("idna")
-                    self.putheader('Host', _strip_ipv6_iface(netloc_enc))
+                    self.putheader('Host', netloc_enc)
                 else:
                     if self._tunnel_host:
                         host = self._tunnel_host
@@ -1249,9 +1172,9 @@ class HTTPConnection:
 
                     # As per RFC 273, IPv6 address should be wrapped with []
                     # when used as Host header
-                    host_enc = self._wrap_ipv6(host_enc)
-                    if ":" in host:
-                        host_enc = _strip_ipv6_iface(host_enc)
+
+                    if host.find(':') >= 0:
+                        host_enc = b'[' + host_enc + b']'
 
                     if port == self.default_port:
                         self.putheader('Host', host_enc)
@@ -1476,15 +1399,43 @@ else:
 
         default_port = HTTPS_PORT
 
-        def __init__(self, host, port=None,
-                     *, timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
-                     source_address=None, context=None, blocksize=8192):
+        # XXX Should key_file and cert_file be deprecated in favour of context?
+
+        def __init__(self, host, port=None, key_file=None, cert_file=None,
+                     timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
+                     source_address=None, *, context=None,
+                     check_hostname=None, blocksize=8192):
             super(HTTPSConnection, self).__init__(host, port, timeout,
                                                   source_address,
                                                   blocksize=blocksize)
+            if (key_file is not None or cert_file is not None or
+                        check_hostname is not None):
+                import warnings
+                warnings.warn("key_file, cert_file and check_hostname are "
+                              "deprecated, use a custom context instead.",
+                              DeprecationWarning, 2)
+            self.key_file = key_file
+            self.cert_file = cert_file
             if context is None:
-                context = _create_https_context(self._http_vsn)
+                context = ssl._create_default_https_context()
+                # enable PHA for TLS 1.3 connections if available
+                if context.post_handshake_auth is not None:
+                    context.post_handshake_auth = True
+            will_verify = context.verify_mode != ssl.CERT_NONE
+            if check_hostname is None:
+                check_hostname = context.check_hostname
+            if check_hostname and not will_verify:
+                raise ValueError("check_hostname needs a SSL context with "
+                                 "either CERT_OPTIONAL or CERT_REQUIRED")
+            if key_file or cert_file:
+                context.load_cert_chain(cert_file, key_file)
+                # cert and key file means the user wants to authenticate.
+                # enable TLS 1.3 PHA implicitly even for custom contexts.
+                if context.post_handshake_auth is not None:
+                    context.post_handshake_auth = True
             self._context = context
+            if check_hostname is not None:
+                self._context.check_hostname = check_hostname
 
         def connect(self):
             "Connect to a host on a given (SSL) port."
@@ -1535,7 +1486,8 @@ class IncompleteRead(HTTPException):
             e = ''
         return '%s(%i bytes read%s)' % (self.__class__.__name__,
                                         len(self.partial), e)
-    __str__ = object.__str__
+    def __str__(self):
+        return repr(self)
 
 class ImproperConnectionState(HTTPException):
     pass
