@@ -1,4 +1,4 @@
-#!/usr/pkg/bin/python3.6
+#!/usr/pkg/bin/python3.7
 """      turtle-example-suite:
 
              tdemo_tree.py
@@ -16,7 +16,7 @@ the current pen is cloned. So in the end
 there are 1024 turtles.
 """
 from turtle import Turtle, mainloop
-from time import clock
+from time import perf_counter as clock
 
 def tree(plist, l, a, f):
     """ plist is list of pens
